/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Id: barvinok_base.cc 9253 2009-05-27 14:34:21Z paffenho $"


#include <cstdlib>
#include <Rational.h>
#include <Integer.h>
#include <polymake.h>
#include <Matrix.h>
#include <iostream>

#include <barvinok/polylib.h>
#include <barvinok/barvinok.h>
#include <barvinok/util.h>
#include <barvinok_convert.h>

#define MAXRAYS	    0

namespace polymake { namespace polytope {
  
  
  // compute the Ehrhart coefficients via barvinok 
  // input is a polyhedron in polylib format
  Vector<Rational> barvinok_ehrhart_polynomial_coeff( Polyhedron *A,  const int d ) {
    
    Vector<Rational> ehrhart_poynomial_coeff(d+1); //create a zero vector of appropriate length
    
    Polyhedron *C = Cone_over_Polyhedron(A);       // cone over the polytope
    Polyhedron *U = Universe_Polyhedron(1);        // needed by barvinok, no parameters involved
    
    // call barvinok
    // returns an evalue representing a piecewise quasi-polynomial
    evalue *EP = barvinok_enumerate_ev(C, U, MAXRAYS);
    
    Polyhedron_Free(C);                        // free
    Polyhedron_Free(U);                        // --"-
    
    // check that we got a piecewise quasi-polynomial
    // we actually only have polynomial, so it should have only one piece (i.e. size =2)
    if ( !value_zero_p(EP->d) || EP->x.p->type != partition || EP->x.p->size != 2 ) {
      evalue_free(EP);
      throw std::runtime_error("barvinok did not return a polynomial");
    }
    
    evalue *poly = &EP->x.p->arr[1]; // extract the polynomial
    
    if (value_notzero_p(poly->d)) {  // if the polynomial is a constant
      ehrhart_poynomial_coeff[0] = Integer(poly->x.n);
    } else {                         // should have a real polynomial
      if ( poly->x.p->type != polynomial ) {
	evalue_free(EP);
	throw std::runtime_error("barvinok did not return a polynomial");
      }
      int l = poly->x.p->size - 1;                       // assign the coefficients
      for (int i = 0; i <= l; ++i) 
	ehrhart_poynomial_coeff[i] = Rational(poly->x.p->arr[i].x.n, poly->x.p->arr[i].d );
    }
    evalue_free(EP);                                     // free
    return ehrhart_poynomial_coeff;
  }
  
  
  // transform an inequality description into polylib format 
  // and call barvinok_ehrhart_polynomial_coeff to compute Ehrhart polynomial coeff
  Vector<Rational> barvinok_ehrhart_polynomial_coeff_facets( const Matrix<Integer> F, const Matrix<Integer> A, const int d ) {
    
    ::Matrix *M = polymake_constraints2polylib(F,A);  // constraints in polylib format
    Polyhedron *P = Constraints2Polyhedron(M, 0);     // the polyhedron
    Matrix_Free(M);                                   // we don't need the matrix anymore
    
    // get the coeffs
    Vector<Rational> ehrhart_poynomial_coeff = barvinok_ehrhart_polynomial_coeff ( P, d );
    Polyhedron_Free(P);                             // free polyhedron
    return ehrhart_poynomial_coeff;
  }
  
  // transform a vertex description into polylib format 
  // and call barvinok_ehrhart_polynomial_coeff to compute Ehrhart polynomial coeff
  Vector<Rational> barvinok_ehrhart_polynomial_coeff_verts( const Matrix<Rational> V, const int d ) {
    
    ::Matrix *M = polymake_vertices2polylib(V);     // vertices in polylib format
    Polyhedron *A = Rays2Polyhedron(M,0);           // the polyhedron
    Matrix_Free(M);                                 // we don't need the matrix anymore
    
    // get the coeffs
    Vector<Rational> ehrhart_poynomial_coeff = barvinok_ehrhart_polynomial_coeff ( A, d );
    
    Polyhedron_Free(A);                             // free polyhedron
    return ehrhart_poynomial_coeff;
  }
  
  
  // computes the number of lattice points of a polytope in polylib description
  // using the barvinok library
  Integer   barvinok_n_lattice_points( Polyhedron *P ) {
    Value cb;
    value_init(cb);
    barvinok_count(P, &cb, 0);
    Integer count(cb);
    value_clear(cb);
    return count;
  }
  
  // computes the number of lattice points of a polytope in inequality description
  // using the barvinok library
  Integer   barvinok_n_lattice_points_from_facets(Matrix<Integer>  F, Matrix<Integer> A) {
    
    ::Matrix *M = polymake_constraints2polylib(F,A);    // the constraints in polylib format
    Polyhedron *P = Constraints2Polyhedron(M, 0);       // the polyhedron
    Matrix_Free(M);                                     // don't need the matrix anymore
    Integer count = barvinok_n_lattice_points( P );     // count using barvinok
    Polyhedron_Free(P);                                 // free the poly
    return count;
  }
  
  // computes the number of lattice pointqs of a polytope in vertex description
  // using the barvinok library
  Integer   barvinok_n_lattice_points_from_verts(Matrix<Rational>  V) {
    
    ::Matrix *M = polymake_vertices2polylib(V);         // the vertices in polylib format
    Polyhedron *Q = Rays2Polyhedron(M, 0);              // the polyhedron
    Matrix_Free(M);                                     // don't need the matrix anymore
    Integer count = barvinok_n_lattice_points( Q );     // count using barvinok
    Polyhedron_Free(Q);                                 // free the poly
    return count;
  }
  
  
  FunctionTemplate4perl("barvinok_n_lattice_points_from_facets(Matrix<Integer>,Matrix<Integer>)");
  FunctionTemplate4perl("barvinok_n_lattice_points_from_verts(Matrix<Rational>)");
  FunctionTemplate4perl("barvinok_ehrhart_polynomial_coeff_facets(Matrix<Integer>,Matrix<Integer>,$)");
  FunctionTemplate4perl("barvinok_ehrhart_polynomial_coeff_verts(Matrix<Rational>,$)");
} }
